<?php
namespace VendorName\ModuleName\Model\GDPR;

use Plumrocket\GDPR\Api\DataProcessorInterface;
use Plumrocket\GDPR\Helper\CustomerData;
use Magento\Customer\Api\Data\CustomerInterface;
use VendorName\ModuleName\Model\ResourceModel\DataModelName\CollectionFactory;

/**
 * Class ProcessorName
 *
 * @package VendorName\ModuleName\Model\GDPR
 */
class ProcessorName implements DataProcessorInterface
{
    /**
     * @var CustomerData
     */
    protected $customerData;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * ProcessorName constructor.
     *
     * @param CustomerData      $customerData
     * @param CollectionFactory $collectionFactory
     * @param array             $dataExport
     * @param array             $dataAnonymize
     */
    public function __construct(
        CustomerData $customerData,
        CollectionFactory $collectionFactory,
        array $dataExport = [],
        array $dataAnonymize = []
    )
    {
        $this->customerData = $customerData;
        $this->collectionFactory = $collectionFactory;
        $this->dataExport = $dataExport;
        $this->dataAnonymize = $dataAnonymize;
    }

    /**
     * This method is used to export customer data based on arguments defined in di.xml file
     *
     * Expected return structure:
     * array(
     * array('HEADER1', 'HEADER2', 'HEADER3', ...),
     * array('VALUE1', 'VALUE2', 'VALUE3', ...),
     * ...
     * )
     *
     * @param CustomerInterface $customer
     *
     * @return array
     */
    public function export(CustomerInterface $customer)
    {
        $customerId = $customer->getId();
        // Retrieve your customer data from database
        $collection = $this->collectionFactory->create()->addFieldToFilter('customer_id', $customerId);
        $returnData = [];
        $i = 0;

        if (! $collection->getSize()) {
            return null;
        }

        foreach ($this->dataExport as $key => $title) {
            $returnData[$i][] = $title;
        }
        $i++;

        foreach ($collection as $item) {
            $itemData = $item->getData();
            foreach ($this->dataExport as $key => $title) {
                $returnData[$i][] = (isset($itemData[$key]) ? $itemData[$key] : '');
            }
            $i++;
        }

        return $returnData;
    }

    /**
     * Executed upon customer data deletion.
     *
     * @param CustomerInterface $customer
     *
     * @return void
     * @throws \Exception
     */
    public function delete(CustomerInterface $customer)
    {
        $customerId = $customer->getId();
        // Retrieve your customer data from database
        $collection = $this->collectionFactory->create()
            ->addFieldToFilter('customer_id', $customerId);
        if (! $collection->getSize()) {
            return;
        }
        $collection->walk('delete');
    }

    /**
     * Executed upon customer data anonymization.
     *
     * @param CustomerInterface $customer
     *
     * @return void
     * @throws \Exception
     */
    public function anonymize(CustomerInterface $customer)
    {
        $customerId = $customer->getId();
        // Retrieve your customer data from database
        $collection = $this->collectionFactory->create()
            ->addFieldToFilter('customer_id', ['eq' => $customerId]);

        $dataAnonymized = $this->customerData->getDataAnonymized($this->dataAnonymize, $customerId);
        if (! empty($dataAnonymized) && $collection->getSize()) {
            $collection->setDataToAll($dataAnonymized)->save();
        }
    }
}
